@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: Jun  7 2021 (14:57) 
 ## Version: 
-## Last-Updated: Jun  7 2021 (14:58) 
+## Last-Updated: apr 14 2023 (17:06) 
 ##           By: Brice Ozenne
-##     Update #: 3
+##     Update #: 8
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -18,7 +18,18 @@
 ## * model.frame.lmm
 ##' @export
 model.frame.lmm <- function(formula, ...){
-    return(formula$data)
+    data <- formula$data
+    rownames(data) <- NULL
+    return(data)
+}
+
+## * model.frame.lmmCC
+##' @export
+model.frame.lmmCC <- function(formula, ...){
+    data <- formula$data
+    keep.var <- unique(manifest(formula, original = TRUE))
+    rownames(data) <- NULL
+    return(data[keep.var])
 }
 
 ##----------------------------------------------------------------------
